/*
 * 05/28/2003
 *
 * camerafreeplanet.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "camerafreeplanet.h"

// ########################
// Classe CCameraFreePlanet
// ########################


// ========================
// Constructeur
// Initialise :
// position = (0, 0, 0)
// cible = (0, 1, 0.5)
// "up" vecteur = (0, 0, 1)
// ========================
CCameraFreePlanet::CCameraFreePlanet(id_t id, char * name) : CCamera(id, name)
{
	m_type = CAMERA_FREE_PLANET;
}


// ===========
// Destructeur
// ===========
CCameraFreePlanet::~CCameraFreePlanet() { }


// ============================================================
// Effectue une rotation de l'angle de vue par rapport a un axe
// ============================================================
void CCameraFreePlanet::RotateView(float angle, float x, float y, float z)
{
	CVector vNewView;

	// Calcul le vecteur direction (la direction dans laquelle la camera regarde)
	CVector vView = m_vView - m_vPosition;		

	// Calcul le sinus et le cosinus de l'angle
	float cosTheta = (float)cos(angle);
	float sinTheta = (float)sin(angle);

	// http://astronomy.swin.edu.au/~pbourke/geometry/rotate/
	// Calcul le nouveau vecteur representant la direction de la camera

	// Find the new x position for the new rotated point
	vNewView.x  = (cosTheta + (1 - cosTheta) * x * x)		* vView.x;
	vNewView.x += ((1 - cosTheta) * x * y - z * sinTheta)	* vView.y;
	vNewView.x += ((1 - cosTheta) * x * z + y * sinTheta)	* vView.z;

	// Find the new y position for the new rotated point
	vNewView.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* vView.x;
	vNewView.y += (cosTheta + (1 - cosTheta) * y * y)		* vView.y;
	vNewView.y += ((1 - cosTheta) * y * z - x * sinTheta)	* vView.z;

	// Find the new z position for the new rotated point
	vNewView.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* vView.x;
	vNewView.z += ((1 - cosTheta) * y * z + x * sinTheta)	* vView.y;
	vNewView.z += (cosTheta + (1 - cosTheta) * z * z)		* vView.z;

	// On additionne le nouveau vecteur de direction a la position
	// de la camera pour avoir la nouvelle cible de la camera
	m_vView = m_vPosition + vNewView;
}


// ==============================================================================
// Change l'angle de vue de la camera en fonction des deplacements de la souris
// x,y correspondent a la position courante de la souris
// lx,ly correspondent a la derniere position de la souris (avant le deplacement)
// ==============================================================================
void CCameraFreePlanet::SetViewByMouse(int x, int y, int lx, int ly, float speed)
{
	float angleY = 0.0f;
	float angleZ = 0.0f;
	static float currentRotX = 0.0f;
	
	// si la souris n'a pas bouge on ne fait rien
	if( (x == lx) && (y == ly) ) return;

	// On determine les angles de rotation suivant les deplacements
	// de la souris et la vitesse "speed"
	angleY = (float)( (lx - x) ) / speed;		
	angleZ = (float)( (ly - y) ) / speed;
	
	// Ici on garde la rotation suivant X
	currentRotX -= angleZ;  
	
	// On teste la rotation X pour eviter de faire des rotations de plus de 180
	if(currentRotX > 1.4f)
		currentRotX = 1.4f;
	else if(currentRotX < -1.4f)
		currentRotX = -1.4f;

	else
	{
		// La rotation X (haut et bas) correspond a une rotation autour
		// de l'axe perpendiculaire au vecteur direction et au vecteur "up"
		CVector vAxis = (m_vView - m_vPosition) ^ m_vUpVector;
		vAxis = CVector::Normalize(vAxis);

		// Rotation autour de l'axe perpendiculaire (haut, bas)
		// et autour de l'axe Y (gauche, droite)
		RotateView(angleZ, vAxis.x, vAxis.y, vAxis.z);
		RotateView(angleY, 0, 0, 1);
	}
}


// =================
// Dessine la camera
// =================
void CCameraFreePlanet::Draw()
{
	glBindTexture(GL_TEXTURE_2D, 0);
	glPushMatrix();
	glColor3f(1.0,1.0,1.0);
	glBegin(GL_LINES);
		glVertex3f(m_vPosition.x, m_vPosition.y, m_vPosition.z);
		glVertex3f(m_vView.x, m_vView.y, m_vView.z);
	glEnd();
	glTranslatef(m_vPosition.x, m_vPosition.y, m_vPosition.z);
	glutWireSphere(0.5, 10, 10);
	glPopMatrix();
}


// =======================================
// Mise a jour de la position de la camera
// =======================================
void CCameraFreePlanet::Update()
{
	static float rotation = 0;
	m_vPosition.x = m_planet->GetPosition().x;
	m_vPosition.y = m_planet->GetPosition().y;
	m_vPosition.z = m_planet->GetPosition().z;

	RotateView(-rotation,m_vUpVector.x, m_vUpVector.y, m_vUpVector.z);
	rotation = DEGTORAD(m_planet->GetRotationAngle());
	RotateView(rotation,m_vUpVector.x, m_vUpVector.y, m_vUpVector.z);
}


// =====================================
// Change la planete suivi par la camera
// ====================================
void CCameraFreePlanet::SetPlanet(CPlanet *planet)
{
	m_planet = planet;
}
