/*
 * 05/28/2003
 *
 * planet.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "planet.h"

// ##############
// Classe CPlanet
// ##############


// Objet quadric utilise pour dessiner les spheres
GLUquadricObj *CPlanet::QUADRIC = gluNewQuadric();

// Timer global
CTimerGLUT *CPlanet::TIMER_GLUT = NULL;

// Classe permettant d'effectuer des dessins bitmaps
CBitmapGL *CPlanet::BITMAP_GL = NULL;

// Facteur permettant de regler la vitesse de rotation
float CPlanet::ROTSPEED_FACTOR = INIT_ROTSPEED_FACTOR;

// Facteur permettant de regler la vitesse de revolution
float CPlanet::REVSPEED_FACTOR = INIT_REVSPEED_FACTOR;

// Affiche les ellipses representant les orbites des planetes
bool CPlanet::ELLIPSES = false;


// ========================
// Constructeur
// id = identifiant
// name = nom de la planete
// ========================
CPlanet::CPlanet(id_t id, char *name) : CObject(id+1, name)
{
	m_type = PLANET;
	char texture[50];
	glGenTextures(1, &m_texture);
	strcpy(texture, GFX_DIR);
	strcat(texture, name);
	strcat(texture, ".tga");
	InitTexture(texture);

	strcpy(texture, GFX_DIR);
	strcat(texture, name);
	strcat(texture, "_symb.tga");
	m_symb = new CTga();
	if (m_symb->Load(texture) != TGA_OK)
		log("Unable to load \"%s\" texture\n", texture);

	m_radius = 10.0f;
	m_distsun = 0.0f;
	m_rotationAngle = 0.0f;
	m_rotationSpeed = 0.0f;
	m_incline = 0.0f;

	m_revolutionAngle = 0.0;
}


// =======================================================================
// Initialise les donnees pour calculer l'ellipse decrite autour du soleil
// =======================================================================
void CPlanet::SetEllipse()
{
	m_c = (m_excentricity*m_distsun)/(1-m_excentricity);
	m_a = m_c/m_excentricity;
	m_b = sqrt(m_a*m_a-m_c*m_c);

	for (int i=0; i<ELLIPSE_RESOLUTION; i++)
	{
		m_ellipseX[i] = -GetNewX((i*(M_PI))/ELLIPSE_RESOLUTION);
		m_ellipseY[i] = GetNewY((i*(M_PI))/ELLIPSE_RESOLUTION);
	}
}


// ================================================================
// Retourne la nouvelle position X de la planete apres une rotation
// dans certain angle autour du soleil
// ================================================================
float CPlanet::GetNewX(float angle)
{
	float newX = 0.0;
	newX = m_a*cos(angle)-m_c;  // on soustrait m_c car changement de repre, le soleil est le centre et non le centre de l'ellipse (a change rien pour Y)
	return newX * 0.5;
}
 

// ================================================================
// Retourne la nouvelle position Y de la planete apres une rotation
// dans certain angle autour du soleil
// ================================================================
float CPlanet::GetNewY(float angle)
{ 
	float newY = 0.0;
	newY = m_b*sin(angle);
	return newY * 0.5;
}


// ===========
// Destructeur
// ===========
CPlanet::~CPlanet()
{
	glDeleteTextures(1, &m_texture);
}


// ==================
// Dessine la planete
// ==================
void CPlanet::Draw()
{
	#if DEBUG_MODE > 0
		CObject::Draw();
	#endif

	// Debut du dessin de l'objet selectionnable
	glLoadName(m_id);
	
	glDisable(GL_LIGHTING);
	glBindTexture(GL_TEXTURE_2D, 0);
	glColor3f(0.1, 0.6, 0.05);

	// Lorsque l'on dessine un objet selectionnable
	// il faut mettre les balises glBegin et glEnd
	glBegin(GL_LINE_STRIP);
	if (ELLIPSES) {
		for (int i=0; i<ELLIPSE_RESOLUTION; i++)
		{
			glVertex3f(m_ellipseX[i], m_ellipseY[i], m_vPosition.z);
		}
		glEnd();
		glBegin(GL_LINE_STRIP);
		for (int j=0; j<ELLIPSE_RESOLUTION; j++)
		{
			glVertex3f(m_ellipseX[j], -m_ellipseY[j], m_vPosition.z);
		}
		glVertex3f(m_ellipseX[ELLIPSE_RESOLUTION-1], m_ellipseY[ELLIPSE_RESOLUTION-1], m_vPosition.z);
	}
	glEnd();
	glEnable(GL_LIGHTING);

	glCullFace(GL_BACK);
	glPushMatrix();

	m_revolutionAngle += ((REVSPEED_FACTOR*0.25)/m_revolutionSpeed) * (TIMER_GLUT->FrameInterval() / 15.0f);
	if (m_revolutionAngle > 360.0f || m_revolutionAngle < -360.0f) m_revolutionAngle = 0.0f;

	m_vPosition.x = -GetNewX(-m_revolutionAngle);
	m_vPosition.y = GetNewY(-m_revolutionAngle);
	
	glTranslatef(m_vPosition.x, m_vPosition.y, m_vPosition.z);

	m_rotationAngle += (float)(((ROTSPEED_FACTOR*100000.0)/m_rotationSpeed) * (TIMER_GLUT->FrameInterval() / 15.0f));
	if (m_rotationAngle > 360.0f || m_rotationAngle < -360.0f) m_rotationAngle = 0.0f;

	glRotatef(-m_incline, 0.0f, 1.0f, 0.0f);
	glRotatef(m_rotationAngle, 0.0f, 0.0f, 1.0f);

	glBindTexture(GL_TEXTURE_2D, m_texture);
	gluSphere(QUADRIC, (m_radius*PLANETS_SCALE), 35, 35);

	glPopMatrix();

	glEnd();	// Fin du dessin de l'objet selectionnable
}


// =======================================
// Dessine le soleil (eclairage different)
// =======================================
void CPlanet::DrawSun()
{
	#if DEBUG_MODE > 0
		CObject::Draw();
	#endif

	// Debut du dessin de l'objet selectionnable
	glLoadName(m_id);

	// Lorsque l'on dessine un objet selectionnable
	// il faut mettre les balises glBegin et glEnd
	glBegin(GL_LINES);							
	glEnd();

	glCullFace(GL_BACK);
	glEnable(GL_LIGHT1);
	glDisable(GL_LIGHT0);

	glPushMatrix();

	glTranslatef(m_vPosition.x, m_vPosition.y, m_vPosition.z);

	m_rotationAngle += (float)(((ROTSPEED_FACTOR*100000.0)/m_rotationSpeed) * (TIMER_GLUT->FrameInterval() / 15.0f));
	if (m_rotationAngle > 360.0f || m_rotationAngle < -360.0f) m_rotationAngle = 0.0f;

	glRotatef(m_rotationAngle, 0.0f, 0.0f, 1.0f);

	glBindTexture(GL_TEXTURE_2D, m_texture);
	gluSphere(QUADRIC, (m_radius*SUN_SCALE), 30, 30);
	
	glPopMatrix();

	glEnable(GL_LIGHT0);
	glDisable(GL_LIGHT1);

	glEnd();	// Fin du dessin de l'objet selectionnable
}


// =====================================================
// Dessine la planete en mode Zoom avec ces informations
// =====================================================
void CPlanet::DrawZoom()
{
	#if DEBUG_MODE > 0
		CObject::Draw();
	#endif

	static float rot = 0.0;

	glCullFace(GL_BACK);

	glTranslatef(7.0f, 3.0f, -40.0f);
	glRotatef(-90, 1.0f, 0.0f, 0.0f);
	glRotatef(10, 0.0f, 1.0f, 0.0f);

	glPushMatrix();

	rot += (float)((TIMER_GLUT->FrameInterval() / 35.0f));
	if (rot > 360.0f) rot = 0.0f;
	
	glRotatef(rot, 0.0f, 0.0f, 1.0f);

	glBindTexture(GL_TEXTURE_2D, m_texture);
	gluSphere(QUADRIC, 10.0f, 40, 40);
	
	glPopMatrix();

	BITMAP_GL->SetTextColor(0.8f, 0.8f, 0.8f);
	BITMAP_GL->Begin2D();
	BITMAP_GL->DrawImage(15,305,m_symb);
	BITMAP_GL->DrawString(45,310,GLUT_BITMAP_HELVETICA_12,"%s", m_name);
	BITMAP_GL->DrawString(15,303,GLUT_BITMAP_HELVETICA_12,"______________");
	BITMAP_GL->DrawString(15,285,GLUT_BITMAP_HELVETICA_12,"Diametre : %.2f km", m_radius*2.0f);
	BITMAP_GL->DrawString(15,270,GLUT_BITMAP_HELVETICA_12,"Masse : %.2f 10^24 kg", m_mass);
	BITMAP_GL->DrawString(15,255,GLUT_BITMAP_HELVETICA_12,"Distance au soleil : %.2f 10^6 km", m_distsun);
	BITMAP_GL->DrawString(15,240,GLUT_BITMAP_HELVETICA_12,"Inclinaison de l'equateur : %.2f degres", m_incline);
	BITMAP_GL->DrawString(15,225,GLUT_BITMAP_HELVETICA_12,"Inclinaison de l'orbite : %.2f degres", m_orbitIncline);
	BITMAP_GL->DrawString(15,210,GLUT_BITMAP_HELVETICA_12,"Excentricite de l'orbite : %.3f", m_excentricity);
	BITMAP_GL->DrawString(15,195,GLUT_BITMAP_HELVETICA_12,"Duree de la rotation siderale : %.2f h", (float)(m_rotationLength)/60.0f);
	BITMAP_GL->DrawString(15,180,GLUT_BITMAP_HELVETICA_12,"Duree de la revolution siderale : %.2f an(s)", (float)(m_revolutionLength)/365.0f);
	BITMAP_GL->DrawString(15,165,GLUT_BITMAP_HELVETICA_12,"Densite (eau=1) : %.1f", m_density);
	BITMAP_GL->DrawString(15,150,GLUT_BITMAP_HELVETICA_12,"Gravite en surface (terre=1) : %.2f", m_gravity);
	BITMAP_GL->DrawString(15,135,GLUT_BITMAP_HELVETICA_12,"Temperature moyenne : %.2f degres C", m_temperature);
	BITMAP_GL->DrawString(15,120,GLUT_BITMAP_HELVETICA_12,"Constituants atmospheriques : %s", m_constituents);
	BITMAP_GL->DrawString(15,105,GLUT_BITMAP_HELVETICA_12,"Nombre de satellite(s) : %d", m_nbSatellites);
	BITMAP_GL->End2D();
}


// =====================
// Initialise la texture
// =====================
void CPlanet::InitTexture(char * name_img)
{
	CTga tga;
	if (tga.Load(name_img) == TGA_OK)
		tga.CreateTexture(m_texture, MIPMAPPING);
	else log("Unable to load \"%s\" texture\n", name_img);
}


// ===============
// Change le rayon
// ===============
void CPlanet::SetRadius(float radius)
{
	m_radius = radius;
}


// =================
// Retourne le rayon
// =================
float CPlanet::GetRadius()
{
	return m_radius;
}

// ====================
// Change l'inclinaison
// ====================
void CPlanet::SetIncline(float incline)
{
	m_incline = incline;
}

// ======================
// Retourne l'inclinaison
// ======================
float CPlanet::GetIncline()
{
	return m_incline;
}


// ========================================
// Change la distance par rapport au soleil
// ========================================
void CPlanet::SetDistsun(float distsun)
{
	m_distsun = distsun;
	m_vPosition.x += (m_distsun);

	m_ellipseX = new float[ELLIPSE_RESOLUTION];
	m_ellipseY = new float[ELLIPSE_RESOLUTION];
}


// ==========================================
// Retourne la distance par rapport au soleil
// ==========================================
float CPlanet::GetDistsun()
{
	return m_distsun;
}


// ===============================================
// Change la vitesse de rotation autour de son axe
// ===============================================
void CPlanet::SetRotationSpeed(float rotationSpeed)
{
	m_rotationSpeed = rotationSpeed;
}


// =================================================
// Retourne la vitesse de rotation autour de son axe
// =================================================
float CPlanet::GetRotationSpeed()
{
	return m_rotationSpeed;
}


// ============================================
// Change l'angle de rotation autour de son axe
// ============================================
void CPlanet::SetRotationAngle(float rotationAngle)
{
	m_rotationAngle = rotationAngle;
}


// ==============================================
// Retourne l'angle de rotation autour de son axe
// ==============================================
float CPlanet::GetRotationAngle()
{
	return m_rotationAngle;
}


// =============================================
// Change l'angle de revolution autour du soleil
// =============================================
void CPlanet::SetRevolutionAngle(float revolutionAngle)
{
	m_revolutionAngle = revolutionAngle;
}


// ===============================================
// Retourne l'angle de revolution autour du soleil
// ===============================================
float CPlanet::GetRevolutionAngle()
{
	return m_revolutionAngle;
}


// =============================
// Change la masse de la planete
// =============================
void CPlanet::SetMass(float mass)
{
	m_mass = mass;
}


// ==============================
// Retourne la mass de la planete
// ==============================
float CPlanet::GetMass()
{ 
	return m_mass;
}


// ===================================
// Retourne l'excentricite de l'orbite
// ===================================
float CPlanet::GetExcentricity()
{
	return m_excentricity;
}


// ===================================
// Retourne l'excentricite de l'orbite
// ===================================
void CPlanet::SetExcentricity(float excentricity)
{
	m_excentricity = excentricity;
}


// ====================================================
// Retourne la duree de la rotation siderale en minutes
// ====================================================
int CPlanet::GetRotationLength()
{
	return m_rotationLength;
}


// ==================================================
// Change la duree de la rotation siderale en minutes
// ==================================================
void CPlanet::SetRotationLength(int rotationLength)
{
	m_rotationLength = rotationLength;
	m_rotationSpeed = m_rotationLength; // ROTATION_EARTH ;
}


// ====================================================
// Retourne la duree de la revolution siderale en jours
// ====================================================
float CPlanet::GetRevolutionLength()
{
	return m_revolutionLength;
}


// ==================================================
// Change la duree de la revolution siderale en jours
// ==================================================
void CPlanet::SetRevolutionLength(float revolutionLength)
{
	m_revolutionLength = revolutionLength;
	m_revolutionSpeed = m_revolutionLength / REVOLUTION_EARTH;
}


// ===================
// Retourne la densite
// ===================
float CPlanet::GetDensity()
{
	return m_density;
}


// =================
// Change la densite
// =================
void CPlanet::SetDensity(float density)
{
	m_density = density;
}


// ===================
// Retourne la gravite
// ===================
float CPlanet::GetGravity()
{
	return m_gravity;
}


// =================
// Change la gravite
// =================
void CPlanet::SetGravity(float gravity)
{
	m_gravity = gravity;
}


// ==================================
// Retourne l'inclinaison de l'orbite
// ==================================
float CPlanet::GetOrbitIncline()
{
	return m_orbitIncline;
}


// ================================
// Change l'inclinaison de l'orbite
// ================================
void CPlanet::SetOrbitIncline(float orbitIncline)
{
	m_orbitIncline = orbitIncline;
}


// =======================
// Retourne la temperature
// =======================
float CPlanet::GetTemperature()
{
	return m_temperature;
}


// =====================
// Change la temperature
// =====================
void CPlanet::SetTemperature(float temperature)
{
	m_temperature = temperature;
}


// ========================================
// Retourne les constituants atmospheriques
// ========================================
char * CPlanet::GetConstituents()
{
	return m_constituents;
}


// ======================================
// Change les constituants atmospheriques
// ======================================
void CPlanet::SetConstituents(char * constituents)
{
	strcpy(m_constituents, constituents);
}


// ================================
// Retourne le nombre de satellites
// ================================
int CPlanet::GetNbSatellites()
{
	return m_nbSatellites;
}


// ==============================
// Change le nombre de satellites
// ==============================
void CPlanet::SetNbSatellites(int nbSatellites)
{
	m_nbSatellites = nbSatellites;
}

