/*
 * 05/28/2003
 *
 * planet.h
 * Classe definissant une planete du systeme solaire
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _PLANET_H
#define _PLANET_H

#include <GL/glut.h>
#include <cstring>
#include "object.h"
#include "tga.h"
#include "error.h"
#include "timerGLUT.h"
#include "bitmapGL.h"

// Echelles
#define PLANETS_SCALE 0.001f
#define PLANETS_SCALE_TXT "1/1000"
#define SUN_SCALE (1.0f/30000.0f)
#define SUN_SCALE_TXT "1/30000"
#define DISTANCE_SCALE_TXT "1/10000000"

#define INIT_ROTSPEED_FACTOR 0.02f
#define INIT_REVSPEED_FACTOR 0.02f
#define MIN_SPEED_FACTOR 0.00125f
#define MAX_SPEED_FACTOR 0.6f

#define M_EARTH 5.98			//*10^24
#define M_SUN 333000*M_EARTH	//*10^24

#define REVOLUTION_EARTH 365
#define ROTATION_EARTH 1436

#define ELLIPSE_RESOLUTION (((int)(10000.0/((int)(m_distsun)))) + 100)

// ##############
// Classe CPlanet
// ##############
class CPlanet : public CObject
{
public:
	
	// Objet quadric utilise pour dessiner les spheres
	static GLUquadricObj *QUADRIC;

	// Timer global
	static CTimerGLUT *TIMER_GLUT;

	// Classe permettant d'effectuer des dessins bitmaps
	static CBitmapGL *BITMAP_GL;

	// Facteur permettant de regler la vitesse de rotation
	static float ROTSPEED_FACTOR;

	// Facteur permettant de regler la vitesse de revolution
	static float REVSPEED_FACTOR;

	// Affiche les ellipses representant les orbites des planetes
	static bool ELLIPSES;

	// ========================
	// Constructeur
	// id = identifiant
	// name = nom de la planete
	// ========================
	CPlanet(id_t id, char *name);

	// ===========
	// Destructeur
	// ===========
	~CPlanet();

	// ==================
	// Dessine la planete
	// ==================
	void Draw();

	// =======================================
	// Dessine le soleil (eclairage different)
	// =======================================
	void DrawSun();

	// =====================================================
	// Dessine la planete en mode Zoom avec ces informations
	// =====================================================
	void DrawZoom();

	// ===============
	// Change le rayon
	// ===============
	void SetRadius(float radius);

	// =================
	// Retourne le rayon
	// =================
	float GetRadius();

	// ====================
	// Change l'inclinaison
	// ====================
	void SetIncline(float incline);

	// ======================
	// Retourne l'inclinaison
	// ======================
	float GetIncline();
	// ========================================
	// Change la distance par rapport au soleil
	// ========================================
	void SetDistsun(float distsun);

	// ==========================================
	// Retourne la distance par rapport au soleil
	// ==========================================
	float GetDistsun();

	// ===============================================
	// Change la vitesse de rotation autour de son axe
	// ===============================================
	void SetRotationSpeed(float rotationSpeed);

	// =================================================
	// Retourne la vitesse de rotation autour de son axe
	// =================================================
	float GetRotationSpeed();

	// ============================================
	// Change l'angle de rotation autour de son axe
	// ============================================
	void SetRotationAngle(float rotationAngle);

	// ==============================================
	// Retourne l'angle de rotation autour de son axe
	// ==============================================
	float GetRotationAngle();

	// =============================================
	// Change l'angle de revolution autour du soleil
	// =============================================
	void SetRevolutionAngle(float revolutionAngle);

	// ===============================================
	// Retourne l'angle de revolution autour du soleil
	// ===============================================
	float GetRevolutionAngle();

	// =============================
	// Change la masse de la planete
	// =============================
	void SetMass(float mass);

	// ==============================
	// Retourne la mass de la planete
	// ==============================
	float GetMass();

	// ===================================
	// Retourne l'excentricite de l'orbite
	// ===================================
	float GetExcentricity();

	// =================================
	// Change l'excentricite de l'orbite
	// =================================
	void SetExcentricity(float excentricity);

	// ====================================================
	// Retourne la duree de la rotation siderale en minutes
	// ====================================================
	int GetRotationLength();

	// ==================================================
	// Change la duree de la rotation siderale en minutes
	// ==================================================
	void SetRotationLength(int rotationLength);

	// ====================================================
	// Retourne la duree de la revolution siderale en jours
	// ====================================================
	float GetRevolutionLength();

	// ==================================================
	// Change la duree de la revolution siderale en jours
	// ==================================================
	void SetRevolutionLength(float revolutionLength);

	// ===================
	// Retourne la densite
	// ===================
	float GetDensity();

	// =================
	// Change la densite
	// =================
	void SetDensity(float density);

	// ===================
	// Retourne la gravite
	// ===================
	float GetGravity();

	// =================
	// Change la gravite
	// =================
	void SetGravity(float gravity);

	// ==================================
	// Retourne l'inclinaison de l'orbite
	// ==================================
	float GetOrbitIncline();

	// ================================
	// Change l'inclinaison de l'orbite
	// ================================
	void SetOrbitIncline(float orbitIncline);

	// =======================
	// Retourne la temperature
	// =======================
	float GetTemperature();

	// =====================
	// Change la temperature
	// =====================
	void SetTemperature(float temperature);

	// ========================================
	// Retourne les constituants atmospheriques
	// ========================================
	char * GetConstituents();

	// ======================================
	// Change les constituants atmospheriques
	// ======================================
	void SetConstituents(char * constituents);

	// ================================
	// Retourne le nombre de satellites
	// ================================
	int GetNbSatellites();

	// ==============================
	// Change le nombre de satellites
	// ==============================
	void SetNbSatellites(int nbSatellites);

	// =======================================================================
	// Initialise les donnees pour calculer l'ellipse decrite autour du soleil
	// =======================================================================
	void SetEllipse();

	// ================================================================
	// Retourne la nouvelle position X de la planete apres une rotation
	// dans certain angle autour du soleil
	// ================================================================
	float GetNewX(float angle);

	// ================================================================
	// Retourne la nouvelle position Y de la planete apres une rotation
	// dans certain angle autour du soleil
	// ================================================================
	float GetNewY(float angle);


private:

	// =====================
	// Initialise la texture
	// =====================
	void InitTexture(char * name_img);

	// Image representant le symbole de la planete
	CTga *m_symb;
	// Identifiant de la texture
	GLuint m_texture;
	// Angle de rotation de la planete sur son axe
	float m_rotationAngle;
	// Vitesse de rotation autour de son axe
	float m_rotationSpeed;
	// Vitesse de revolution autour du soleil
	float m_revolutionSpeed;
	// Demi grand axe de l'ellipse dcrite par la plante
	float m_a;   
	// Demi petit axe de l'ellipse dcrite par la plante
	float m_b;  
    // Distance centre de l'ellipse-foyer (soleil)
	float m_c;    
	// Angle du mouvement elliptique de la plante
	float m_revolutionAngle;  
	// Coordonnees X,Y pour dessiner l'ellipse
	float *m_ellipseX;
	float *m_ellipseY;
	
	// Rayon
	float m_radius;
	// Distance par rapport au soleil
	float m_distsun;
	// Inclinaison de l'equateur sur l'orbite
	float m_incline;
	// Masse
	float m_mass;
	// Excentricite de l'orbite	
	float m_excentricity;
	// Duree de la rotation sidrale en minutes
	int m_rotationLength;
	// Duree de la revolution sidrale en jours
	float m_revolutionLength;
	// Densite
	float m_density;
	// Gravite
	float m_gravity;
	// Inclinaison de l'orbite
	float m_orbitIncline;
	// Temperature
	float m_temperature;
	// Constituants atmospheriques
	char m_constituents[20];
	// Nombre de satellites
	int m_nbSatellites;
};

#endif
